/*
 * Working with files

 */
package com.sea.utils;

import com.sea.log.Logger;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;


/**
 *
 * @author vorisek
 */
public class FileUtils {

// --------------------------------------------------
// Public Constants
// --------------------------------------------------
// --------------------------------------------------
// Public Properties
// --------------------------------------------------
public static int iLogLevel = Logger.WARN;

// --------------------------------------------------
// Private Constants
// --------------------------------------------------
// --------------------------------------------------
// Private Properties
// --------------------------------------------------
// --------------------------------------------------
// Public Methods
// --------------------------------------------------
/**
 * Read whole file into a String
 * 
 * @param sFilename, example "file:///A:/ADFiles/calibration.xml"
 * @return Whole content of the file or empty string 
 */
public static String readTextFile(String sFilename) {

  InputStream is = null;
  FileConnection fc = null;
  String result = "";

  try {
    log("Readfing file '" + sFilename + "'.");
    fc = (FileConnection) Connector.open(sFilename, Connector.READ_WRITE);

    if (fc.exists()) {
      int size = (int) fc.fileSize();
      is = fc.openInputStream();
      byte bytes[] = new byte[size];
      is.read(bytes, 0, size);
      result = new String(bytes, 0, size);
      log("Content '" + getPreview(result, 400) + "'.");
    }
  } catch (IOException ioe) {
    // There is an error while opening or reading the file. Return empty string.
    log("[EXCEPTION] - " + ioe.getMessage(), Logger.ERROR);
  } finally {
    try {
      if (null != is) {
        is.close();
      }
      if (null != fc) {
        fc.close();
      }
    } catch (IOException e) {
      System.out.println(e.getMessage());
    }
  }
  return result;
}

/**
 * Write whole file at once
 */
public static void writeTextFile(String sFilename, String sText) {
  OutputStream os = null;
  FileConnection fconn = null;
  try {
    log("Writing file '" + sFilename + "' with content '" + getPreview(sText, 400) + "'.");
    fconn = (FileConnection) Connector.open(sFilename, Connector.READ_WRITE);
    if (!fconn.exists()) {
      fconn.create();
    }

    os = fconn.openDataOutputStream();
    os.write(sText.getBytes());
    fconn.setHidden(false);
//          fconn.setReadable(true);
  } catch (IOException e) {
    log("[EXCEPTION] - " + e.getMessage(), Logger.ERROR);
  } finally {
    try {
      if (null != os) {
        os.close();
      }
      if (null != fconn) {
        fconn.close();
      }
    } catch (IOException e) {
      System.out.println(e.getMessage());
    }
  }
}


/**
 * Delete a file
 * 
 * @param sFilename 
 */
public static void delete(String sFilename) {
  FileConnection fin = null;
  log("Detele file '" + sFilename + "' ... ");
  try {
    fin = (FileConnection) Connector.open(sFilename);
    if (fin.exists()) {
      fin.delete();
      log("[  OK  ] - deleted");
    } else {
      log("[  OK  ] - does not exists");
    }
    fin.close();

  } catch (IOException e) {
    log("[EXCEPTION] - " + e.getMessage(), Logger.ERROR);
  }
}


// --------------------------------------------------
// Private Methods
// --------------------------------------------------
/**
 * Debug printing
 */
private static void log(String line) {
  log(line, Logger.INFO);
}

/**
 * Debug printing
 */
private static void log(String line, int level) {
  if (iLogLevel >= level) {
    Logger.log("@File", line);
  }
}

/**
 * Return first max. 'len' character from String 'src'
 * 
 * @param src
 * @param len
 * @return String
 */
private static String getPreview(String src, int len) {
  if (len < src.length()) {
    return src.substring(0, len);
  } else {
    return src;
  }
}

}
